/***********************************************************************/
/*  This file is part of the ARM Toolchain package                     */
/*  Copyright (c) 2010 Keil - An ARM Company. All rights reserved.     */
/***********************************************************************/
/*                                                                     */
/*  FlashDev.C:  Flash Programming Functions adapted                   */
/*               for SCMCU SC32F2xxA Series Flash                       */
/*                                                                     */
/***********************************************************************/

#include "..\FlashOS.H"        // FlashOS Structures

typedef volatile unsigned long    vu32;
typedef          unsigned long     u32;

// Flash Memory Controller (FMC)
typedef struct
{
  vu32 CON;
  vu32 ADR;
  vu32 DAT;
  vu32 CMD;
  vu32 LOCK;
} FMC_TypeDef;

typedef struct
{
  vu32            DID;
  vu32             AHBCKDIV;
  vu32             APBCKDIV;
  vu32             APBCKEN;
  vu32            CLKODIV;
  vu32             PCON;
  vu32             RSTCON;
  vu32             RSTSTAT;
  vu32             CLKCON;
  vu32             CLKSEL;
  vu32             CLKSTAT;
  vu32             APBCKSEL;
  vu32             IOMUX;
  vu32             CIDL;
  vu32            CIDH;
  vu32            LVDCFG;

} SYSCON_TypeDef;

// Flash memory map
#define FMC_BASE            (0x49800000UL)
#define AHB_BASE            (0x50000000UL)                            /*!< (AHB     ) Base Address */
#define SYSCON_BASE         (AHB_BASE)  

// Peripheral Declaration
#define FMC                 ((FMC_TypeDef       *) FMC_BASE)
#define SYSCON              ((SYSCON_TypeDef    *) SYSCON_BASE)       /*!< SYSCON configuration struct       */

void oscint(void)
{
 	vu32 dly;
    SYSCON->CLKSEL = 	0X5A690002;
	for(dly=0; dly<10; dly++);	
	SYSCON->CLKCON = 0X5A690001;	
	for(dly=0; dly<10; dly++);
	SYSCON->CLKCON = 0X5A690008;	

	for(dly=0; dly<10; dly++);
	SYSCON->CLKSEL = 	0X5A690000;
}


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
    
  oscint();
  FMC->LOCK = 0x55AA6699;
  if(FMC->LOCK != 1)
    return(1);
  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {
  while(FMC->CON & 0x20) {};
  FMC->LOCK = 0;
  if(FMC->LOCK != 0)
    return(1);
  return (0);                                  // Finished without Errors
}

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {
	while(FMC->CON & 0x20) {};
  FMC->CMD = 0x06;
  while(FMC->CON & 0x20) {};
  return (0);                                  // Finished without Errors
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {
  while(FMC->CON & 0x20) {};
  FMC->ADR = adr;
  FMC->DAT = 0x00;  
  FMC->CMD = 0x03;
  while(FMC->CON & 0x20) {};
  return (0);                                  // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
	
  sz = (sz + 3) & ~3;                          // Adjust size for Words
  while(sz) {
    FMC->ADR = (adr & ~0x00000003);            // Prepare Address	
		FMC->DAT = *((u32 *)buf);                  // Prepare Data			

    while(FMC->CON & 0x20);
    FMC->CMD = 0x02;                           // Exacute Command
    adr += 4;
    buf += 4;
    sz  -= 4;
  }
	
  return (0);                                  // Finished without Errors
}


/*
 *  Verify Flash Contents
 *    Parameter:      adr:  Start Address
 *                    sz:   Size (in bytes)
 *                    buf:  Data
 *    Return Value:   (adr+sz) - OK, Failed Address
 */

unsigned long Verify (unsigned long adr, unsigned long sz, unsigned char *buf) {
  sz = (sz + 3) & ~3;                          // Adjust size for Words
  while (sz) {
    FMC->ADR = (adr & ~0x00000003);            // Prepare Address
		while(FMC->CON & 0x20);
    FMC->CMD = 0x1;                            // set command
	  while(FMC->CON & 0x20);
			if (FMC->DAT	!= *((u32 *)buf)) {          // Compare Data
				return (adr);
			}  			
		
    adr += 4;                                  // Go to next Word
    buf += 4;
    sz  -= 4;
  }
  return (adr);
}
